SystemOrganization addCategory: #'Neo-Unicode-Tests'!

TestResource subclass: #NeoUnicodeNormalizationTestResource
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Unicode-Tests'!

!NeoUnicodeNormalizationTestResource commentStamp: 'SvenVanCaekenberghe 12/11/2015 10:41' prior: 0!
I hold the contents of NormalizationTest.txt loaded from a URL.

http://www.unicode.org/Public/UNIDATA/NormalizationTest.txt

# Normalization Test Suite
# Format:
#
#   Columns (c1, c2,...) are separated by semicolons
#   They have the following meaning:
#      source; NFC; NFD; NFKC; NFKD
#   Comments are indicated with hash marks
#   Each of the columns may have one or more code points.
#
# CONFORMANCE:
# 1. The following invariants must be true for all conformant implementations
#
#    NFC
#      c2 ==  toNFC(c1) ==  toNFC(c2) ==  toNFC(c3)
#      c4 ==  toNFC(c4) ==  toNFC(c5)
#
#    NFD
#      c3 ==  toNFD(c1) ==  toNFD(c2) ==  toNFD(c3)
#      c5 ==  toNFD(c4) ==  toNFD(c5)
#
#    NFKC
#      c4 == toNFKC(c1) == toNFKC(c2) == toNFKC(c3) == toNFKC(c4) == toNFKC(c5)
#
#    NFKD
#      c5 == toNFKD(c1) == toNFKD(c2) == toNFKD(c3) == toNFKD(c4) == toNFKD(c5)
#
# 2. For every code point X assigned in this version of Unicode that is not specifically
#    listed in Part 1, the following invariants must be true for all conformant
#    implementations:
#
#      X == toNFC(X) == toNFD(X) == toNFKC(X) == toNFKD(X)!

!NeoUnicodeNormalizationTestResource methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/11/2015 09:48'!
data
	"A collection of dictionaries with keys: #source #NFC #NFD #NFKC #NFKD and #comment"
	
	^ data! !

!NeoUnicodeNormalizationTestResource methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 12/11/2015 10:20'!
parseLine: line
	| columns test comment |
	columns := $; split: line.
	test := SmallDictionary new.
	#(#source #NFC #NFD #NFKC #NFKD)
		doWithIndex: [ :key :number | 
			| value |
			value := columns at: number.
			value := $  split: value.
			value := value collect: [ :each | Integer readFrom: each base: 16 ] as: Array.
			test at: key put: value ].
	comment := line copyFrom: (line indexOf: $#) + 2 to: line size.
	test at: #comment put: comment.
	^ test! !

!NeoUnicodeNormalizationTestResource methodsFor: 'running' stamp: 'SvenVanCaekenberghe 12/11/2015 10:21'!
setUp
	data := Array streamContents: [ :out | 
		self url retrieveContents linesDo: [ :line | 
			(line isEmpty or: [ line first = $# or: [ line first = $@ ] ])
				ifFalse: [ 
					| test |
					test := self parseLine: line.
					out nextPut: test ] ] ]! !

!NeoUnicodeNormalizationTestResource methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/11/2015 10:05'!
url
	^ 'http://www.unicode.org/Public/UNIDATA/NormalizationTest.txt' asZnUrl! !

TestCase subclass: #NeoUnicodeCaserTests
	instanceVariableNames: 'caser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Unicode-Tests'!

!NeoUnicodeCaserTests methodsFor: 'running' stamp: 'SvenVanCaekenberghe 12/8/2015 20:22'!
setUp
	caser := NeoUnicodeCaser new! !

!NeoUnicodeCaserTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/8/2015 20:24'!
testLowercase
	self assert: (caser case: #lowercase string: 'ABC') equals: 'abc'.
	self assert: (caser case: #lowercase string: '123 !!= abc') equals: '123 !!= abc'.! !

!NeoUnicodeCaserTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/8/2015 20:24'!
testUppercase
	self assert: (caser case: #uppercase string: 'abc') equals: 'ABC'.
	self assert: (caser case: #uppercase string: '123 !!= ABC') equals: '123 !!= ABC'.! !

TestCase subclass: #NeoUnicodeCharacterDataTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Unicode-Tests'!

!NeoUnicodeCharacterDataTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/8/2015 15:25'!
testDigitOne
	| unicodeCharacterData |
	unicodeCharacterData := $1 unicodeCharacterData.
	self assert: unicodeCharacterData codePoint equals: 49.
	self assert: unicodeCharacterData character equals: $1.
	self assert: unicodeCharacterData name equals: 'DIGIT ONE'.
	self assert: unicodeCharacterData isNumber.
	self deny: unicodeCharacterData isLetter.
	self assert: unicodeCharacterData isNumberDecimal.
	self assert: unicodeCharacterData decimalDigitValue equals: 1! !

!NeoUnicodeCharacterDataTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/8/2015 15:25'!
testLatinCapitalLetterA
	| unicodeCharacterData |
	unicodeCharacterData := $A unicodeCharacterData.
	self assert: unicodeCharacterData codePoint equals: 65.
	self assert: unicodeCharacterData character equals: $A.
	self assert: unicodeCharacterData name equals: 'LATIN CAPITAL LETTER A'.
	self assert: unicodeCharacterData isLetter.
	self deny: unicodeCharacterData isNumber.
	self assert: unicodeCharacterData isUppercase.
	self assert: unicodeCharacterData lowercase equals: $a codePoint! !

!NeoUnicodeCharacterDataTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/8/2015 15:25'!
testLatinSmallLetterA
	| unicodeCharacterData |
	unicodeCharacterData := $a unicodeCharacterData.
	self assert: unicodeCharacterData codePoint equals: 97.
	self assert: unicodeCharacterData character equals: $a.
	self assert: unicodeCharacterData name equals: 'LATIN SMALL LETTER A'.
	self assert: unicodeCharacterData isLetter.
	self deny: unicodeCharacterData isNumber.
	self assert: unicodeCharacterData isLowercase.
	self assert: unicodeCharacterData uppercase equals: $A codePoint! !

!NeoUnicodeCharacterDataTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/8/2015 13:49'!
testLatinSmallLetterEWithAcute
	| unicodeCharacterData |
	unicodeCharacterData := $é unicodeCharacterData.
	"LATIN SMALL LETTER E WITH ACUTE [U+00E9]"
	self assert: unicodeCharacterData codePoint equals: 233.
	self assert: unicodeCharacterData character equals: $é.
	self assert: unicodeCharacterData name equals: 'LATIN SMALL LETTER E WITH ACUTE'.
	self assert: unicodeCharacterData isLetter.
	self deny: unicodeCharacterData isNumber.
	self assert: unicodeCharacterData isLowercase.
	self assert: unicodeCharacterData uppercase equals: $É codePoint.
	self assert: unicodeCharacterData hasStandardCanonicalDecomposition.
	"LATIN SMALL LETTER E [U+0065] followed by COMBINING ACUTE ACCENT [U+0301]"
	self assert: unicodeCharacterData decompositionMapping equals: #(101 769)! !

TestCase subclass: #NeoUnicodeNormalizerConformanceTests
	instanceVariableNames: 'normalizer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Unicode-Tests'!

!NeoUnicodeNormalizerConformanceTests commentStamp: 'SvenVanCaekenberghe 12/11/2015 23:08' prior: 0!
This test uses http://www.unicode.org/Public/UNIDATA/NormalizationTest.txt to run about 75.000 individual test cases to test conformance to the official Unicode Normalization specification.!

!NeoUnicodeNormalizerConformanceTests class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/11/2015 22:39'!
resources
	^ Array with: NeoUnicodeNormalizationTestResource ! !

!NeoUnicodeNormalizerConformanceTests methodsFor: 'running' stamp: 'SvenVanCaekenberghe 12/11/2015 22:37'!
setUp
	normalizer := NeoUnicodeNormalizer new! !

!NeoUnicodeNormalizerConformanceTests methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/11/2015 22:48'!
specification
	^ NeoUnicodeNormalizationTestResource current data! !

!NeoUnicodeNormalizerConformanceTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/11/2015 22:55'!
testNFC
	| failed succeeded |
	failed := OrderedCollection new.
	succeeded := 0.
	self specification do: [ :spec | | result |
		result := Array streamContents: [ :out |
			normalizer composeFrom: (spec at: #source) readStream to: out ].
		result = (spec at: #NFC)
			ifTrue: [ succeeded := succeeded + 1 ]
			ifFalse: [ failed add: spec -> result ] ].
	self assert: failed isEmpty! !

!NeoUnicodeNormalizerConformanceTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/11/2015 22:59'!
testNFD
	| failed succeeded |
	failed := OrderedCollection new.
	succeeded := 0.
	self specification do: [ :spec | | result |
		result := Array streamContents: [ :out |
			normalizer decomposeFrom: (spec at: #source) readStream to: out ].
		result = (spec at: #NFD)
			ifTrue: [ succeeded := succeeded + 1 ]
			ifFalse: [ failed add: spec -> result ] ].
	self assert: failed isEmpty! !

!NeoUnicodeNormalizerConformanceTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/11/2015 22:58'!
testNFKC
	| failed succeeded |
	failed := OrderedCollection new.
	succeeded := 0.
	self specification do: [ :spec | | result |
		result := Array streamContents: [ :out |
			normalizer composeCompatibleFrom: (spec at: #source) readStream to: out ].
		result = (spec at: #NFKC)
			ifTrue: [ succeeded := succeeded + 1 ]
			ifFalse: [ failed add: spec -> result ] ].
	self assert: failed isEmpty! !

!NeoUnicodeNormalizerConformanceTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/11/2015 23:19'!
testNFKD
	| failed succeeded |
	failed := OrderedCollection new.
	succeeded := 0.
	self specification do: [ :spec | | result |
		result := Array streamContents: [ :out |
			normalizer decomposeCompatibleFrom: (spec at: #source) readStream to: out ].
		result = (spec at: #NFKD)
			ifTrue: [ succeeded := succeeded + 1 ]
			ifFalse: [ failed add: spec -> result ] ].
	self assert: failed isEmpty! !

TestCase subclass: #NeoUnicodeNormalizerTests
	instanceVariableNames: 'normalizer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Unicode-Tests'!

!NeoUnicodeNormalizerTests methodsFor: 'running' stamp: 'SvenVanCaekenberghe 12/11/2015 23:21'!
setUp
	normalizer := NeoUnicodeNormalizer new! !

!NeoUnicodeNormalizerTests methodsFor: 'tests' stamp: 'SvenVanCaekenberghe 12/8/2015 13:43'!
testFundamentalComposition
	| in out |
	"LATIN SMALL LETTER E [U+0065] followed by COMBINING ACUTE ACCENT [U+0301]"
	in := #(101 769).
	out := Array streamContents: [ :writeStream |
		normalizer composeFrom: in readStream to: writeStream ].
	"LATIN SMALL LETTER E WITH ACUTE [U+00E9]"
	self assert: out equals: #(233)! !

!NeoUnicodeNormalizerTests methodsFor: 'tests' stamp: 'SvenVanCaekenberghe 12/8/2015 13:43'!
testFundamentalDecomposition
	| in out |
	"LATIN SMALL LETTER E WITH ACUTE [U+00E9]"
	in := #(233).
	out := Array streamContents: [ :writeStream |
		normalizer decomposeFrom: in readStream to: writeStream ].
	"LATIN SMALL LETTER E [U+0065] followed by COMBINING ACUTE ACCENT [U+0301]"
	self assert: out equals: #(101 769).! !

!NeoUnicodeNormalizerTests methodsFor: 'tests' stamp: 'SvenVanCaekenberghe 12/8/2015 12:53'!
testSimple
	| original composed decomposed |
	original := 'les élèves Français'.
	decomposed := normalizer decomposeString: original.
	self assert: decomposed equals: 'les élèves Français'.
	composed := normalizer composeString: decomposed.
	self assert: composed equals: original.
	original := 'Düsseldorf Königsallee'.
	decomposed := normalizer decomposeString: original.
	self assert: decomposed equals: 'Düsseldorf Königsallee'.
	composed := normalizer composeString: decomposed.
	self assert: composed equals: original.
	! !
